#include <assert.h>
#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>

#include "windef.h"
#include "winbase.h"
#include "wingdi.h"
#include "winuser.h"

#define CW_USEDEFAULT16 ((WORD)0x8000)

extern const char *SPY_GetMsgName(UINT msg);

/*
WM_MDIREFRESHMENU is sent by USER32 to mdiclient:
1. after WM_PARENTNOTIFY/WM_CREATE and before placing a new child on the top.
2. after hiding a child and removing focus from it,
   before WM_PARENTNOTIFY/WM_DESTROY and before sending WM_DESTROY,
   WM_NCDESTROY to a destroyed child, WM_MDIDESTROY to mdiclient.
mdiclient refreshes "window" menu by adding/removing children.
*/

static const char mdi_lParam_test_message[] = "just a test string";

static WNDPROC old_mdiclient_proc;
static HHOOK hhook;

static LRESULT CALLBACK cbt_hook_proc(int nCode, WPARAM wParam, LPARAM lParam) 
{ 
    static const char *CBT_code_name[10] = {
        "HCBT_MOVESIZE",
        "HCBT_MINMAX",
        "HCBT_QS",
        "HCBT_CREATEWND",
        "HCBT_DESTROYWND",
        "HCBT_ACTIVATE",
        "HCBT_CLICKSKIPPED",
        "HCBT_KEYSKIPPED",
        "HCBT_SYSCOMMAND",
        "HCBT_SETFOCUS" };
    const char *code_name = (nCode >= 0 && nCode <= HCBT_SETFOCUS) ? CBT_code_name[nCode] : "Unknown";

    printf("CBT: %d (%s), %08x, %08lx\n", nCode, code_name, wParam, lParam);

    switch (nCode)
    {
        case HCBT_CREATEWND:
        {
            CBT_CREATEWNDA *createwnd = (CBT_CREATEWNDA *)lParam;
            printf("HCBT_CREATEWND: hwnd %p, parent %p, style %08lx\n",
                  (HWND)wParam, createwnd->lpcs->hwndParent, createwnd->lpcs->style);
            assert(createwnd->hwndInsertAfter == HWND_TOP);// "hwndInsertAfter should be always HWND_TOP\n");
            break;
        }
    }

    return CallNextHookEx(hhook, nCode, wParam, lParam);
}

#define ENTER(x) printf("mdiclient: " #x " enter\n")
#define LEAVE(x) printf("mdiclient: " #x " leave\n")

static LRESULT CALLBACK mdiclient_hook_proc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    LRESULT ret;

    /* do not log mouse messages */
    if ((message >= WM_MOUSEFIRST && message <= WM_MOUSELAST) ||
        (message == WM_NCHITTEST) ||
        (message == WM_SETCURSOR) ||
        (message >= WM_IME_SETCONTEXT && message <= WM_IME_KEYUP))
        return CallWindowProc(old_mdiclient_proc, hWnd, message, wParam, lParam);

//printf("parent window has %d items in its Windows menu\n",
//        GetMenuItemCount(GetSubMenu(GetMenu(GetParent(hWnd)), 1)));

//    printf("GetWindowLongA(0) = %08lx\n", GetWindowLongPtrA(hWnd, 0));
//    printf("GetWindowLongA(4) = %08lx\n", GetWindowLongPtrA(hWnd, 4));
//    printf("GetWindowLongA(GWL_USERDATA) = %08lx\n", GetWindowLongPtrA(hWnd, GWLP_USERDATA));

    switch (message)
    {
    case WM_NCCREATE:
    case WM_CREATE:
        {
            CREATESTRUCT *cs = (CREATESTRUCT *)lParam;

            if (message == WM_CREATE) ENTER(WM_CREATE);
            else ENTER(WM_NCCREATE);

            printf("initial size %d,%d - %d,%d\n",
                cs->x, cs->y, cs->x + cs->cx, cs->y + cs->cy);
        }
        break;
    case WM_SIZE:
        printf("mdiclient: WM_SIZE enter (width %d, height %d)\n", LOWORD(lParam), HIWORD(lParam));
        break;
    case WM_WINDOWPOSCHANGING:
    {
        WINDOWPOS *winpos = (WINDOWPOS *)lParam;
        ENTER(WM_WINDOWPOSCHANGING);
        printf("%p after %p, x %d, y %d, cx %d, cy %d flags %08x\n",
                winpos->hwnd, winpos->hwndInsertAfter,
                winpos->x, winpos->y, winpos->cx, winpos->cy, winpos->flags);
        break;
    }
    case WM_WINDOWPOSCHANGED:
    {
        WINDOWPOS *winpos = (WINDOWPOS *)lParam;
        ENTER(WM_WINDOWPOSCHANGED);
        printf("%p after %p, x %d, y %d, cx %d, cy %d flags %08x\n",
                winpos->hwnd, winpos->hwndInsertAfter,
                winpos->x, winpos->y, winpos->cx, winpos->cy, winpos->flags);
        break;
    }
    case WM_TIMER:
        ENTER(WM_TIMER);
        break;
    case WM_CLOSE:
        ENTER(WM_CLOSE);
        break;
    case WM_DESTROY:
        ENTER(WM_DESTROY);
        break;
    case WM_SETTEXT:
        printf("mdiclient: WM_SETTEXT %s\n", (LPSTR)lParam);
        break;
    case WM_PARENTNOTIFY:
        ENTER(WM_PARENTNOTIFY);
        switch (LOWORD(wParam))
        {
            case WM_CREATE:
                printf("    WM_CREATE child %p\n", (HWND)lParam);
                break;
            case WM_DESTROY:
                printf("    WM_DESTROY child %p\n", (HWND)lParam);
                break;
        }
        break;
    case WM_STYLECHANGING:
        {
            STYLESTRUCT *style = (STYLESTRUCT *)lParam;
            printf("mdiclient: WM_STYLECHANGING %s old %08lx, new %08lx\n",
                wParam == GWL_EXSTYLE ? "Ex" : "", style->styleOld, style->styleNew);
        }
        break;
    case WM_SETFOCUS:
        printf("mdiclient: WM_SETFOCUS: prev focus %p\n", (HWND)wParam);
        break;
    case WM_KILLFOCUS:
        printf("mdiclient: WM_KILLFOCUS: current focus %p\n", (HWND)wParam);
        break;

    case WM_MDICREATE: ENTER(WM_MDICREATE); break;
    case WM_MDIDESTROY: ENTER(WM_MDIDESTROY); break;
    case WM_MDIACTIVATE: ENTER(WM_MDIACTIVATE); break;
    case WM_MDIRESTORE: ENTER(WM_MDIRESTORE); break;
    case WM_MDINEXT: ENTER(WM_MDINEXT); break;
    case WM_MDIMAXIMIZE: ENTER(WM_MDIMAXIMIZE); break;
    case WM_MDITILE: ENTER(WM_MDITILE); break;
    case WM_MDICASCADE: ENTER(WM_MDICASCADE); break;
    case WM_MDIICONARRANGE: ENTER(WM_MDIICONARRANGE); break;
    case WM_MDIGETACTIVE: ENTER(WM_MDIGETACTIVE); break;
    case WM_MDISETMENU: ENTER(WM_MDISETMENU); break;
    case WM_MDIREFRESHMENU:
    {
        HMENU hFrameMenu = GetMenu(GetParent(hWnd));
        printf("parent window has %d items in its menu %p\n",
                GetMenuItemCount(GetSubMenu(hFrameMenu, 1)), hFrameMenu);
        ENTER(WM_MDIREFRESHMENU);
        break;
    }
    case WM_NCACTIVATE: ENTER(WM_NCACTIVATE); break;
    }

    printf("mdiclient: call %08p msg %04x (%s) wparam %08x lparam %08lx\n",
        hWnd, message, SPY_GetMsgName(message), wParam, lParam);
    ret = CallWindowProc(old_mdiclient_proc, hWnd, message, wParam, lParam);
    printf("mdiclient: ret  %08p msg %04x (%s) wparam %08x lparam %08lx, retval %08lx\n",
        hWnd, message, SPY_GetMsgName(message), wParam, lParam, ret);

    switch (message)
    {
    case WM_NCCREATE:
        LEAVE(WM_NCCREATE); break;
    case WM_CREATE:
        LEAVE(WM_CREATE); break;
    case WM_SIZE:
        LEAVE(WM_SIZE); break;
    case WM_WINDOWPOSCHANGING:
        LEAVE(WM_WINDOWPOSCHANGING); break;
    case WM_WINDOWPOSCHANGED:
        LEAVE(WM_WINDOWPOSCHANGED); break;
    case WM_TIMER:
        LEAVE(WM_TIMER); break;
    case WM_CLOSE:
        LEAVE(WM_CLOSE); break;
    case WM_DESTROY:
        LEAVE(WM_DESTROY); break;
    case WM_PARENTNOTIFY:
        LEAVE(WM_PARENTNOTIFY); break;

    case WM_MDICREATE: LEAVE(WM_MDICREATE); break;
    case WM_MDIDESTROY: LEAVE(WM_MDIDESTROY); break;
    case WM_MDIACTIVATE: LEAVE(WM_MDIACTIVATE); break;
    case WM_MDIRESTORE: LEAVE(WM_MDIRESTORE); break;
    case WM_MDINEXT: LEAVE(WM_MDINEXT); break;
    case WM_MDIMAXIMIZE: LEAVE(WM_MDIMAXIMIZE); break;
    case WM_MDITILE: LEAVE(WM_MDITILE); break;
    case WM_MDICASCADE: LEAVE(WM_MDICASCADE); break;
    case WM_MDIICONARRANGE: LEAVE(WM_MDIICONARRANGE); break;
    case WM_MDIGETACTIVE: LEAVE(WM_MDIGETACTIVE); break;
    case WM_MDISETMENU: LEAVE(WM_MDISETMENU); break;
    case WM_MDIREFRESHMENU:
    {
        HMENU hFrameMenu = GetMenu(GetParent(hWnd));
        LEAVE(WM_MDIREFRESHMENU);
        printf("parent window has %d items in its menu %p\n",
                GetMenuItemCount(GetSubMenu(hFrameMenu, 1)), hFrameMenu);
        break;
    }
    case WM_NCACTIVATE: LEAVE(WM_NCACTIVATE); break;
    }

//    printf("GetWindowLongA(0) = %08lx\n", GetWindowLongPtrA(hWnd, 0));
//    printf("GetWindowLongA(4) = %08lx\n", GetWindowLongPtrA(hWnd, 4));
//    printf("GetWindowLongA(GWL_USERDATA) = %08lx\n", GetWindowLongPtrA(hWnd, GWLP_USERDATA));

//printf("parent window has %d items in its Windows menu\n",
//        GetMenuItemCount(GetSubMenu(GetMenu(GetParent(hWnd)), 1)));

    return ret;
}
/*
static void subclass_mdiclient_2(HWND parent)
{
    HWND mdi_client;
    CLIENTCREATESTRUCT client_cs;

    client_cs.hWindowMenu = 0;
    client_cs.idFirstChild = 0;
    
    mdi_client = CreateWindowExA(0, "mdiclient",
                                 NULL,
                                 0,
                                 0, 0, 0, 0,
                                 parent, 0, GetModuleHandle(0),
                                 &client_cs);
    assert(mdi_client);
    old_mdiclient_proc = (WNDPROC)SetClassLong(mdi_client, GCL_WNDPROC, (LONG)mdiclient_hook_proc);
    DestroyWindow(mdi_client);
}
*/

static void subclass_mdiclient(HWND parent)
{
    WNDCLASSA cls;

    if (!GetClassInfoA(0, "MDIClient", &cls)) assert(0);

    old_mdiclient_proc = cls.lpfnWndProc;

    cls.hInstance = GetModuleHandle(0);
    cls.lpfnWndProc = mdiclient_hook_proc;
    cls.lpszClassName = "my_mdiclient";
    if (!RegisterClassA(&cls)) assert(0);
}

static void dump_menu(HMENU hWindowMenu)
{
    UINT i, count;

    count = GetMenuItemCount(hWindowMenu);

    printf("=====================\n");
    printf("current window menu %p has %u items\n", hWindowMenu, count);

    for (i = 0; i < count; i++)
    {
        MENUITEMINFOW mii;
        WCHAR buf[256];

        memset(&mii, 0, sizeof(mii));
        mii.cbSize = sizeof(mii);
        mii.fMask = MIIM_TYPE | MIIM_ID;
        mii.dwTypeData = buf;
        mii.cch = 256;
        GetMenuItemInfoW(hWindowMenu, i, TRUE, &mii);

        if (mii.fType & MFT_SEPARATOR)
            printf("%u: SEPARATOR id %04x\n", i, mii.wID);
	else
            printf("%u: \"%ws\" id %04x\n", i, buf, mii.wID);
    }
    printf("=====================\n");
}

static void test_MDISetMenu(HWND parent, HWND mdi_client, HWND mdi_child)
{
    HMENU hFrameMenu = GetMenu(parent);
    HMENU hOldWindowMenu = GetSubMenu(GetMenu(parent), 1);
    HMENU hWindowMenu;

    SendMessage(mdi_client, WM_MDISETMENU, 0, 0);

    SendMessage(mdi_client, WM_MDIREFRESHMENU, 0, 0);
    printf("Original Old Window Menu %p\n", hOldWindowMenu);
    dump_menu(hOldWindowMenu);

    InsertMenu(hOldWindowMenu, 1, MF_BYPOSITION|MF_STRING, 10, "string 3");
    InsertMenu(hOldWindowMenu, 3, MF_BYPOSITION|MF_SEPARATOR, -1, NULL);
    InsertMenu(hOldWindowMenu, 4, MF_BYPOSITION|MF_SEPARATOR, 1, "string 4");

//    AppendMenu(hOldWindowMenu, MF_STRING, 10, "string 4");
//    AppendMenu(hOldWindowMenu, MF_SEPARATOR, -1, NULL);

    printf("Modified Old Window Menu %p\n", hOldWindowMenu);
    dump_menu(hOldWindowMenu);

//    SendMessage(mdi_client, WM_MDIREFRESHMENU, 0, 0);
//    printf("Refreshed Old Window Menu %p\n", hOldWindowMenu);
//    dump_menu(hOldWindowMenu);

//ShowWindow(mdi_child, SW_HIDE);

    hWindowMenu = CreateMenu();
    AppendMenu(hWindowMenu, MF_STRING, 10, "string 1");
    AppendMenu(hWindowMenu, MF_SEPARATOR, 0, NULL);
    AppendMenu(hWindowMenu, MF_STRING, 10, "string 2");
    AppendMenu(hWindowMenu, MF_SEPARATOR, 0, NULL);

    printf("New Window Menu %p\n", hWindowMenu);
    dump_menu(hWindowMenu);
//    SendMessage(mdi_client, WM_MDISETMENU, 0, (LPARAM)hWindowMenu);
    SendMessage(mdi_client, WM_MDISETMENU, (WPARAM)hFrameMenu, (LPARAM)hWindowMenu);

    printf("Old Window Menu %p\n", hOldWindowMenu);
    dump_menu(hOldWindowMenu);

    printf("Set New Window Menu %p\n", hWindowMenu);
    dump_menu(hWindowMenu);
}

static void test_MDI_create(HWND parent, HWND mdi_client)
{
    MDICREATESTRUCTA mdi_cs;
    HWND mdi_child;

//test_MDISetMenu(parent, mdi_client);

    mdi_cs.szClass = "MDI_child_Class";
    mdi_cs.szTitle = "MDI child 1";
    mdi_cs.hOwner = GetModuleHandle(0);
    mdi_cs.x = 0;
    mdi_cs.y = 0;
    mdi_cs.cx = 100;
    mdi_cs.cy = 100;
    mdi_cs.style = WS_CAPTION | WS_CHILD | WS_VISIBLE;
    mdi_cs.lParam = (LPARAM)mdi_lParam_test_message;
    mdi_child = (HWND)SendMessageA(mdi_client, WM_MDICREATE, 0, (LPARAM)&mdi_cs);
    assert(mdi_child);
    //DestroyWindow(mdi_child);

test_MDISetMenu(parent, mdi_client, mdi_child);
//ShowWindow(mdi_child, SW_HIDE);
//test_MDISetMenu(parent, mdi_client);
//ShowWindow(mdi_child, SW_SHOW);
//test_MDISetMenu(parent, mdi_client);
//ShowWindow(mdi_child, SW_HIDE);
//test_MDISetMenu(parent, mdi_client);
return;


printf("maximizing the child %p\n", mdi_child);
ShowWindow(mdi_child, SW_SHOWMAXIMIZED);
printf("end of maximizing the child %p\n", mdi_child);
printf("restoring the child %p\n", mdi_child);
ShowWindow(mdi_child, SW_RESTORE);
printf("end of restoring the child %p\n", mdi_child);
return;
/*
    mdi_child = CreateMDIWindowA("MDI_child_Class", "MDI child 2",
                                 WS_CAPTION | WS_CHILD | WS_VISIBLE,
                                 10, 10, 100, 100,
                                 mdi_client, GetModuleHandle(0),
                                 (LPARAM)mdi_lParam_test_message);
    assert(mdi_child);
    //DestroyWindow(mdi_child);
*/

    mdi_child = CreateWindowExA(WS_EX_MDICHILD, "MDI_child_Class", "MDI child 3",
                                WS_CAPTION | WS_CHILD | WS_VISIBLE,
                                20, 20, 100, 100,
                                mdi_client, 0, GetModuleHandle(0),
                                (LPVOID)mdi_lParam_test_message);
    assert(mdi_child);
    //DestroyWindow(mdi_child);
return;
    
    mdi_child = CreateWindowExA(0, "MDI_child_Class", "MDI child 4",
                                WS_CAPTION | WS_CHILD | WS_VISIBLE,
                                20, 20, 100, 100,
                                mdi_client, 0, GetModuleHandle(0),
                                (LPVOID)mdi_lParam_test_message);
    assert(mdi_child);
    //DestroyWindow(mdi_child);
}

static LRESULT WINAPI mdi_child_wnd_proc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam)
{
    LRESULT ret;

        /* do not log mouse messages */
    if ((msg >= WM_MOUSEFIRST && msg <= WM_MOUSELAST) ||
        (msg == WM_NCHITTEST) ||
        (msg == WM_SETCURSOR) ||
        (msg >= WM_IME_SETCONTEXT && msg <= WM_IME_KEYUP))
    {
        if (GetWindowLongA(hwnd, GWL_EXSTYLE) & WS_EX_MDICHILD)
	    return DefMDIChildProcA(hwnd, msg, wparam, lparam);
	else
	    return DefWindowProcA(hwnd, msg, wparam, lparam);
    }

    printf("child: Call %08p msg %04x (%s) wparam %08x lparam %08lx\n",
            hwnd, msg, SPY_GetMsgName(msg), wparam, lparam);
//printf("child: parent window has %d items in its Windows menu\n",
//        GetMenuItemCount(GetSubMenu(GetMenu(GetParent(GetParent(hwnd))), 1)));

    switch(msg)
    {
    case WM_PAINT:
        {
            PAINTSTRUCT ps;
            BeginPaint(hwnd, &ps);
            EndPaint(hwnd, &ps);
            return 1;
        }
    case WM_NCCREATE:
    case WM_CREATE:
        {
            CREATESTRUCT *cs = (CREATESTRUCT *)lparam;

            if (msg == WM_CREATE) printf("    WM_CREATE\n");
            else printf("    WM_NCCREATE\n");

            printf("cs size %d,%d - %d,%d\n",
                cs->x, cs->y, cs->cx, cs->cy);

	    if (GetWindowLongA(hwnd, GWL_EXSTYLE) & WS_EX_MDICHILD)
	    {
                MDICREATESTRUCTA *mdi_cs = (MDICREATESTRUCTA *)cs->lpCreateParams;
                printf("mdi_cs size %d,%d - %d,%d\n",
                    mdi_cs->x, mdi_cs->y, mdi_cs->cx, mdi_cs->cy);
	    }
        }
        break;
    case WM_CLOSE:
        printf("    WM_CLOSE\n");
        break;
    case WM_DESTROY:
        printf("    WM_DESTROY\n");
        break;
    case WM_SHOWWINDOW:
        printf("    WM_SHOWWINDOW %08x, %08lx\n", wparam, lparam);
        break;
    case WM_SIZE:
        printf("    WM_SIZE (width %d, height %d)\n", LOWORD(lparam), HIWORD(lparam));
        break;
    case WM_WINDOWPOSCHANGING:
    {
        WINDOWPOS *winpos = (WINDOWPOS *)lparam;
        printf("    WM_WINDOWPOSCHANGING\n");
        printf("%p after %p, x %d, y %d, cx %d, cy %d flags %08x\n",
                winpos->hwnd, winpos->hwndInsertAfter,
                winpos->x, winpos->y, winpos->cx, winpos->cy, winpos->flags);
        break;
    }
    case WM_WINDOWPOSCHANGED:
    {
        WINDOWPOS *winpos = (WINDOWPOS *)lparam;
        printf("    WM_WINDOWPOSCHANGED\n");
        printf("%p after %p, x %d, y %d, cx %d, cy %d flags %08x\n",
                winpos->hwnd, winpos->hwndInsertAfter,
                winpos->x, winpos->y, winpos->cx, winpos->cy, winpos->flags);
        break;
    }
    case WM_GETMINMAXINFO:
        printf("    WM_GETMINMAXINFO\n");
        break;
    case WM_NCCALCSIZE:
        printf("    WM_NCCALCSIZE\n");
        break;
    case WM_CHILDACTIVATE:
        printf("    WM_CHILDACTIVATE\n");
        break;
    case WM_NCACTIVATE:
        printf("    WM_NCACTIVATE\n");
        break;
    case WM_MDIACTIVATE:
        printf("    WM_MDIACTIVATE\n");
        break;
    case WM_SETFOCUS:
        printf("    WM_SETFOCUS: prev focus %p\n", (HWND)wparam);
        break;
    case WM_KILLFOCUS:
        printf("    WM_KILLFOCUS: current focus %p\n", (HWND)wparam);
        break;
    case WM_SETTEXT:
        printf("    WM_SETTEXT %s\n", (LPSTR)lparam);
        break;
    case WM_GETTEXT:
        printf("    WM_GETTEXT\n");
        break;
    }

    if (GetWindowLongA(hwnd, GWL_EXSTYLE) & WS_EX_MDICHILD)
    {
        printf("child: Call DefMDIChildProcA(%p,%04x,%08x,%08lx)\n", hwnd, msg, wparam, lparam);
        ret = DefMDIChildProcA(hwnd, msg, wparam, lparam);
    }
    else
    {
        printf("child: Call DefWindowProcA(%p,%04x,%08x,%08lx)\n", hwnd, msg, wparam, lparam);
        ret = DefWindowProcA(hwnd, msg, wparam, lparam);
    }
//printf("child: parent window has %d items in its Windows menu\n",
//        GetMenuItemCount(GetSubMenu(GetMenu(GetParent(GetParent(hwnd))), 1)));

    switch(msg)
    {
    case WM_GETTEXT:
        printf("    WM_GETTEXT %s\n", (LPSTR)lparam);
        break;
    }

    printf("child: ret  %08p msg %04x (%s) wparam %08x lparam %08lx, retval %08lx\n",
            hwnd, msg, SPY_GetMsgName(msg), wparam, lparam, ret);
    return ret;
}

static LRESULT WINAPI mdi_frame_wnd_proc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam)
{
    static HWND mdi_client;
    LRESULT ret;

    /* do not log mouse messages */
    if ((msg >= WM_MOUSEFIRST && msg <= WM_MOUSELAST) ||
        (msg == WM_NCHITTEST) ||
        (msg == WM_SETCURSOR) ||
        (msg >= WM_IME_SETCONTEXT && msg <= WM_IME_KEYUP))
        return DefFrameProcA(hwnd, mdi_client, msg, wparam, lparam);
        //return DefWindowProcA(hwnd, msg, wparam, lparam);

    switch (msg)
    {
        case WM_CREATE:
        {
            CLIENTCREATESTRUCT client_cs;

//SetWindowPos(hwnd, 0, 0, 0, 300, -90000, SWP_NOMOVE | SWP_NOACTIVATE);
//SetWindowPos(hwnd, 0, 0, 0, 300, 40000, SWP_NOMOVE | SWP_NOACTIVATE);

            subclass_mdiclient(hwnd);
            
            client_cs.hWindowMenu = GetSubMenu(GetMenu(hwnd), 1);
            client_cs.idFirstChild = 1;

//RemoveMenu(client_cs.hWindowMenu, 0, MF_BYPOSITION);

            //mdi_client = CreateWindowExA(0, "mdiclient",
            mdi_client = CreateWindowExA(0, "my_mdiclient",
                                         NULL,
                                         WS_CHILD | WS_CLIPCHILDREN | WS_VISIBLE
                                         | MDIS_ALLCHILDSTYLES,
                                         0, 0, 100, 100,
                                         hwnd, 0, GetModuleHandle(0),
                                         (LPVOID)&client_cs);
            assert(mdi_client);

            {
                WNDCLASS wc;
		DWORD style;

		style = GetWindowLongA(mdi_client, GWL_STYLE);
		printf("mdiclient: style %08lx\n", style);

                if (GetClassInfo(GetModuleHandle(0), "my_mdiclient", &wc))
                {
                    printf("mdiclient: cbClsExtra %d, cbWndExtra %d\n", wc.cbClsExtra, wc.cbWndExtra);
                    printf("GetWindowLongA(0) = %08lx\n", GetWindowLongA(mdi_client, 0));
                    printf("GetWindowLongA(4) = %08lx\n", GetWindowLongA(mdi_client, 4));
                }
            }

//            test_MDI_create(hwnd, mdi_client);
            break;
        }

        case WM_PAINT:
        {
            PAINTSTRUCT ps;
            BeginPaint(hwnd, &ps);
            EndPaint(hwnd, &ps);
            return 1;
        }

        case WM_COMMAND:
            if (LOWORD(wparam) == 100)
            {
                HWND mdi_child;
                HMENU hSysMenu, hmenu_windows = GetSubMenu(GetMenu(hwnd), 1);
                char mdi_child_text[128];
                static int n_mdi_children;

                printf("before MDI child creation Windows menu has %d items\n",
                        GetMenuItemCount(hmenu_windows));

                mdi_child = GetWindow(mdi_client, GW_CHILD);
                printf("top MDI child %p is %szoomed\n",
                        mdi_child, IsZoomed(mdi_child) ? "" : "NOT ");

		////////////////////
#if 0
                wsprintf(mdi_child_text, "MDI child %d", ++n_mdi_children);
                mdi_child = CreateWindowExA(
                                0,
                                "MDI_child_Class", mdi_child_text,
                                WS_CHILD | WS_CAPTION | WS_SYSMENU | WS_CLIPSIBLINGS |
                                WS_MINIMIZEBOX | WS_MAXIMIZEBOX | WS_VISIBLE,
                                CW_USEDEFAULT, 100,
                                200, 200,
                                mdi_client, 0, GetModuleHandle(0),
                                0);
                assert(mdi_child);
		////////////////////
#else
                wsprintf(mdi_child_text, "MDI child %d", ++n_mdi_children);
                mdi_child = CreateWindowExA(
                                WS_EX_MDICHILD,
                                //0,
                                "MDI_child_Class", mdi_child_text,
                                WS_CHILD | WS_CAPTION | WS_SYSMENU | WS_CLIPSIBLINGS |
                                WS_MINIMIZEBOX | WS_MAXIMIZEBOX | WS_VISIBLE,
                                //WS_MINIMIZEBOX | WS_MAXIMIZEBOX,
                                //CW_USEDEFAULT, CW_USEDEFAULT, 200, 200,
                                CW_USEDEFAULT, CW_USEDEFAULT, //100, 100,
                                CW_USEDEFAULT, CW_USEDEFAULT,
                                mdi_client, 0, GetModuleHandle(0),
                                //hwnd, 0, GetModuleHandle(0),
                                0);
                assert(mdi_child);
#endif
                hSysMenu = GetSystemMenu(mdi_child, FALSE);
                AppendMenuA(hSysMenu, MF_SEPARATOR, 0, NULL);
                AppendMenuA(hSysMenu, MF_STRING, -1, mdi_child_text);
/*
		{
                DWORD exStyle;
                exStyle = GetWindowLong(mdi_child, GWL_EXSTYLE);
                printf("old child style %08lx\n", exStyle);
                exStyle |= WS_EX_MDICHILD;
                SetWindowLong(mdi_child, GWL_EXSTYLE, exStyle);
                exStyle = GetWindowLong(mdi_child, GWL_EXSTYLE);
                printf("new child style %08lx\n", exStyle);

                SetWindowText(mdi_child, "new child window text");
		}
*/
                mdi_child = GetWindow(mdi_client, GW_CHILD);
                printf("top MDI child %p is %szoomed\n",
                        mdi_child, IsZoomed(mdi_child) ? "" : "NOT ");

                printf("after MDI child creation Windows menu has %d items\n",
                        GetMenuItemCount(hmenu_windows));
//if (n_mdi_children == 1)
//    AppendMenuA(hmenu_windows, MF_SEPARATOR, -1, NULL);
//AppendMenuA(hmenu_windows, MF_STRING, -1, "another window text");
            }
            break;

        case WM_CLOSE:
            PostQuitMessage(0);
            break;
        case WM_SETTEXT:
            printf("frame: WM_SETTEXT %s\n", (LPSTR)lparam);
            break;
        case WM_GETTEXT:
            printf("frame: WM_GETTEXT\n");
            break;
        case WM_GETMINMAXINFO:
        {
	    MINMAXINFO *minmax = (MINMAXINFO *)lparam;
            printf("frame: WM_GETMINMAXINFO %08x %08lx\n", wparam, lparam);
            printf("ptReserved = (%ld,%ld)\n"
                   "ptMaxSize = (%ld,%ld)\n"
                   "ptMaxPosition = (%ld,%ld)\n"
                   "ptMinTrackSize = (%ld,%ld)\n"
                   "ptMaxTrackSize = (%ld,%ld)\n",
		   minmax->ptReserved.x, minmax->ptReserved.y,
		   minmax->ptMaxSize.x, minmax->ptMaxSize.y,
		   minmax->ptMaxPosition.x, minmax->ptMaxPosition.y,
		   minmax->ptMinTrackSize.x, minmax->ptMinTrackSize.y,
		   minmax->ptMaxTrackSize.x, minmax->ptMaxTrackSize.y);
            break;
        }
        case WM_WINDOWPOSCHANGING:
        {
            WINDOWPOS *winpos = (WINDOWPOS *)lparam;
            printf("frame: WM_WINDOWPOSCHANGING\n");
            printf("%p after %p, x %d, y %d, cx %d, cy %d flags %08x\n",
                    winpos->hwnd, winpos->hwndInsertAfter,
                    winpos->x, winpos->y, winpos->cx, winpos->cy, winpos->flags);
            break;
        }
        case WM_WINDOWPOSCHANGED:
        {
            WINDOWPOS *winpos = (WINDOWPOS *)lparam;
            printf("frame: WM_WINDOWPOSCHANGED\n");
            printf("%p after %p, x %d, y %d, cx %d, cy %d flags %08x\n",
                    winpos->hwnd, winpos->hwndInsertAfter,
                    winpos->x, winpos->y, winpos->cx, winpos->cy, winpos->flags);
            break;
        }
    }

    printf("frame: Call %08p msg %04x (%s) wparam %08x lparam %08lx\n",
            hwnd, msg, SPY_GetMsgName(msg), wparam, lparam);
    ret = DefFrameProcA(hwnd, mdi_client, msg, wparam, lparam);
    //ret = DefWindowProcA(hwnd, msg, wparam, lparam);

    switch (msg)
    {
        case WM_GETTEXT:
            printf("frame: WM_GETTEXT %s\n", (LPSTR)lparam);
            break;
        case WM_GETMINMAXINFO:
        {
	    MINMAXINFO *minmax = (MINMAXINFO *)lparam;
            printf("frame: WM_GETMINMAXINFO %08x %08lx\n", wparam, lparam);
            printf("ptReserved = (%ld,%ld)\n"
                   "ptMaxSize = (%ld,%ld)\n"
                   "ptMaxPosition = (%ld,%ld)\n"
                   "ptMinTrackSize = (%ld,%ld)\n"
                   "ptMaxTrackSize = (%ld,%ld)\n",
		   minmax->ptReserved.x, minmax->ptReserved.y,
		   minmax->ptMaxSize.x, minmax->ptMaxSize.y,
		   minmax->ptMaxPosition.x, minmax->ptMaxPosition.y,
		   minmax->ptMinTrackSize.x, minmax->ptMinTrackSize.y,
		   minmax->ptMaxTrackSize.x, minmax->ptMaxTrackSize.y);
            break;
        }
    }

    printf("frame: ret  %08p msg %04x (%s) wparam %08x lparam %08lx, retval %08lx\n",
            hwnd, msg, SPY_GetMsgName(msg), wparam, lparam, ret);
    return ret;
}

static BOOL mdi_RegisterWindowClasses(void)
{
    WNDCLASSA cls;

    cls.style = 0;
    cls.lpfnWndProc = mdi_frame_wnd_proc;
    cls.cbClsExtra = 0;
    cls.cbWndExtra = 0;
    cls.hInstance = GetModuleHandleA(0);
    cls.hIcon = 0;
    cls.hCursor = LoadCursorA(0, (LPSTR)IDC_ARROW);
    cls.hbrBackground = GetStockObject(WHITE_BRUSH);
    cls.lpszMenuName = MAKEINTRESOURCEA(1);
    cls.lpszClassName = "MDI_parent_Class";
    if(!RegisterClassA(&cls)) return FALSE;

    cls.lpfnWndProc = mdi_child_wnd_proc;
    cls.lpszClassName = "MDI_child_Class";
    if(!RegisterClassA(&cls)) return FALSE;

    return TRUE;
}

int main(void)
{
    HWND mdi_hwndMain;
    MSG msg;
    HACCEL hAccel;

    setvbuf(stdout, NULL, _IONBF, 0);

    if (!mdi_RegisterWindowClasses()) assert(0);

    hhook = SetWindowsHookExA(WH_CBT, cbt_hook_proc, 0, GetCurrentThreadId());
    assert(hhook);

    mdi_hwndMain = CreateWindowExA(0, "MDI_parent_Class", "MDI parent window",
                                   WS_CAPTION | WS_SYSMENU | WS_CLIPCHILDREN |
                                   WS_MINIMIZEBOX | WS_MAXIMIZEBOX |
                                   WS_THICKFRAME | WS_VISIBLE,
                                   100, 100, CW_USEDEFAULT, CW_USEDEFAULT,
                                   GetDesktopWindow(), 0,
                                   GetModuleHandleA(0), NULL);
    assert(mdi_hwndMain);

    hAccel = LoadAccelerators(GetModuleHandleA(0), MAKEINTRESOURCE(1));

    while (GetMessage(&msg, 0, 0, 0))
    {
        if (!TranslateAccelerator(mdi_hwndMain, hAccel, &msg))
        {
            TranslateMessage(&msg);
            DispatchMessage(&msg);
        }
    }

    UnhookWindowsHookEx(hhook);
}
